#include<iostream>
#include<complex>
#define complex complex<double>

#include<bar.h>
#include<fluid.h>
#include<grid.h>
#include<hamop.h>
#include<wavefunction.h>
#include<../base/kbhit.cc> // kbhit() for unix 
//
// potentials, nuclear_charge etc. are defined in potentials.cc
//
#include "potentials.cc"

int main(int argc, char **argv)
{

  char cstr_fname_logfi[1000], cstr_fname_wf_ini[1000], 
       cstr_fname_obser[1000], cstr_fname_wf_fin[1000],
       cstr_fname_info[1000];
  double imag_timestep, time, acc, E_tot, E_tot_prev;
  int iinitmode, i, me, iv, ikey;
  long lno_of_ts, ts;   
  FILE *file_logfi, *file_wf_ini, *file_obser_imag, *file_wf_fin, *file_info;

  complex timestep;

  fluid ells, ms;
  grid g;
  hamop hamilton;
  wavefunction staticpot, E_i, wf;

  fprintf(stdout," CIRC_IM: Imaginary-time propagation for hydrogen atom\n");
  fprintf(stdout," (C) Copyright by Bauer D and Koval P, Heidelberg (2005)\n");
  fprintf(stdout," -------------------------------------------------------\n");

  //
  // input
  //
  // *** declare the grid ***
  g.set_dim(44); // 44 elliptical polariz., 34 linear polariz.
  g.set_ngps(1000,1,1);
  g.set_delt(0.15/nuclear_charge);
  g.set_offs(0,0,0);

  iinitmode = 1; // 1 -- random, 2 -- hydrogenic wf.

  iv        = 1; // verbosity of stdout

  sprintf(cstr_fname_logfi, "./res/circ_im-%d-%ld-%.2le.log",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_wf_ini,"./res/circ_im_wf_ini-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_obser, "./res/circ_im_obser-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_wf_fin,"./res/circ_im_wf_fin-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());
  sprintf(cstr_fname_info,  "./res/circ_im_info-%d-%ld-%.2le.dat",
    g.dimens(), g.ngps_x(), g.delt_x());

  //
  // prepare for propagation ...
  // 

  // number of imaginary time steps
  lno_of_ts = 640000;
  ells.init(g.ngps_z());
  ms.init(g.ngps_z());

  ells[0]             = 0; // populated l quantum number (needed for
                           // initialization only)
  ms[0]               = 0; // populated m quantum number (needed for
                           // initialization only)
  me                  = 0; // dummy here

  E_i.init(g.ngps_z());

  // *** declare rest of variables ***
  imag_timestep=g.delt_x()/4.0;

  // the Hamiltonian
  hamilton.init(g,vecpot_x,vecpot_y,vecpot_z,scalarpotx,scalarpoty,scalarpotz,
                  imagpot,field);

  // this is the linear and constant part of the Hamiltonian
  staticpot.init(g.size()); 
  staticpot.calculate_staticpot(g,hamilton);

  // *** create some files with appropriate appendices
  file_logfi      = fopen(cstr_fname_logfi,"w");
  if (file_logfi==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_logfi); exit(-2);}

  file_obser_imag = fopen(cstr_fname_obser,"w");
  if (file_obser_imag==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_obser); exit(-3);}

  file_wf_ini    = fopen(cstr_fname_wf_ini,"w");
  if (file_wf_ini==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_wf_ini); exit(-4);}

  if (iv!=0)
  {
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_logfi);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_wf_ini);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_obser);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_wf_fin);
     fprintf(stderr, "%s will be (re)written.\n", cstr_fname_info);
  }

  ts=0;
  
  // *** new initialization ***
  // *** the wavefunction array 
  wf.init(g.size()); 
  wf.init_rlm(g,iinitmode,1.0,ells,ms); // appropriate for prop. mode 44
  wf.normalize(g);


  fprintf(stdout,"Norm: %le\n",wf.norm(g));
  
  //
  // write in log file
  //
  fprintf(file_logfi,"Imaginary-time propagation\n");
  fprintf(file_logfi,"Grid: \n");
  fprintf(file_logfi,"g.ngps_x() = %ld\n", g.ngps_x());
  fprintf(file_logfi,"g.ngps_y() = %ld\n", g.ngps_y());
  fprintf(file_logfi,"g.ngps_z() = %ld\n", g.ngps_z());
  fprintf(file_logfi,"g.dimens() = %d\n\n", g.dimens());
  fprintf(file_logfi,"g.delt_x() = %20.15le\n", g.delt_x());

  fprintf(file_logfi,"imag_timestep     = %20.15le\n", imag_timestep);
  fprintf(file_logfi,"lno_of_ts         = %ld\n", lno_of_ts);
  fprintf(file_logfi,"nuclear_charge    = %20.15le\n", nuclear_charge);
  fprintf(file_logfi,"iinitmode         = %d\n", iinitmode);
  fprintf(file_logfi,"cstr_fname_wf_ini = %s\n", cstr_fname_wf_ini);
  fprintf(file_logfi,"cstr_fname_obser  = %s\n", cstr_fname_obser);
  fprintf(file_logfi,"cstr_fname_wf_fin = %s\n", cstr_fname_wf_fin);
  fprintf(file_logfi,"cstr_fname_info   = %s\n", cstr_fname_info);
  fflush(file_logfi);


  // *** the orbitals before relaxation
  wf.dump_to_file_sh(g,file_wf_ini,1);
  fclose(file_wf_ini);

  // ********************************************************
  // ***** imaginary propagation ****************************
  // ********************************************************


  E_tot = 0.0;
  timestep=complex(0.0,-1.0*imag_timestep);  
  time = 0.0;

  printf("Press any key to finish imaginary-time propagation...\n");
  if ((iv==1) && (is_time(0.1)))
fprintf(stdout,
" time                   \
E_tot                  \
accuracy              \
step\n");

  for (ts=0; ts<lno_of_ts; ts++)
  {
    time = time + imag(timestep);  
    //
    // calculate the total energy
    // 
    E_tot_prev = E_tot;
    E_tot = real(wf.energy(0.0,g,hamilton,me,staticpot,nuclear_charge));
    acc = fabs((E_tot_prev-E_tot)/(E_tot_prev+E_tot));
    fprintf(file_obser_imag,"% 20.15le % 20.15le %20.15le %ld\n",
      time,E_tot,acc,ts);

    if ((iv==1) && (is_time(0.1)))
      fprintf(stdout,"% 20.15le % 20.15le %20.15le %ld\n",time,E_tot,acc,ts);

    wf.propagate(timestep, 0.0, g, hamilton, me, staticpot,0, nuclear_charge);

    wf.normalize(g);

    //
    // exclude the following and adjust lno_of_ts 
    // if you have problems with kbhit()
    //
    ikey=kbhit(0);
    if (ikey==-1) {fprintf(stderr, "err: kbhit(0)==-1\n"); exit(1);}
    if (ikey) break;
  }
 

  fprintf(file_logfi, "acc = %le\n", acc);

  fclose(file_obser_imag);
  fclose(file_logfi);


  file_wf_fin    = fopen(cstr_fname_wf_fin,"w");
  if (file_wf_fin==NULL)
  {fprintf(stderr, "%s cant be opened!\n", cstr_fname_wf_fin); exit(-5);}
  wf.dump_to_file_sh(g,file_wf_fin,1);
  fclose(file_wf_fin);

  file_info = fopen(cstr_fname_info, "w");
  fprintf(file_info,"<nuclear_charge>%20.15le</nuclear_charge>\n",
    nuclear_charge);
  fprintf(file_info,"<ngps_x>%ld</ngps_x>\n",g.ngps_x());
  fprintf(file_info,"<ngps_y>%ld</ngps_y>\n",g.ngps_y());
  fprintf(file_info,"<ngps_z>%ld</ngps_z>\n",g.ngps_z());
  fprintf(file_info,"<delt_x>%20.15le</delt_x>\n",g.delt_x());
  fprintf(file_info,"<dimens>%d</dimens>\n",g.dimens());
  fclose(file_info);

  if (iv!=0)
  {
     fprintf(stderr, "%s is written.\n", cstr_fname_logfi);
     fprintf(stderr, "%s is written.\n", cstr_fname_wf_ini);
     fprintf(stderr, "%s is written.\n", cstr_fname_obser);
     fprintf(stderr, "%s is written.\n", cstr_fname_wf_fin);
     fprintf(stderr, "%s is written.\n", cstr_fname_info);
  }

  fprintf(stdout, "Hasta la vista...\n");
}

// 
// end of main program
// 
