double nuclear_charge = 10.0;  // neon
double w=0.2;     // frequ. in au
double E_0=0.0;   // electric field amplitude 
                  // (will be assigned in circ_re.cc)
double n_c=10.0;  // trapezoidal pulse: no. of cycles of constant intensity
                  // sinusoidal pulse: total no. of cycles
double duration;  // will be assigned when vecpot is called
double ampl_im=100.0;  // amplitude of imaginary absorbing potential
double screeningb=2.3227; // screening parameter in the effective potential

//
// functions
//



double vecpot_x(double time, int me)
{
  double ww =w/(2.0*n_c);
  return -E_0/w*sin(ww*time)*sin(ww*time)*(sin(w*time)+sin(2*w*time));
}                  
  

//
double vecpot_y(double time, int me)
{
  double ww =w/(2.0*n_c);
  return E_0/w*sin(ww*time)*sin(ww*time)*(cos(w*time)-cos(2*w*time));
}  


double vecpot_z(double time, int me)
{
  return 0.0;
}


double scalarpotx(double x, double y, double z, double time, int me)
{  

  double result=-(1.0 + (nuclear_charge-1.0)*exp(-screeningb*x))/x;
  return result;
}



double scalarpoty(double x, double y, double z, double time, int me)
{
  double result=0.0;  
  return result;
}


double scalarpotz(double x, double y, double z, double time, int me)
{
  double result=0.0;
  return result;
}
  

double field(double time, int me)
{
  double result=0.0;
  return result;
}  


double imagpot(long xindex, long yindex, long zindex, double time, grid g)
{
  double x,y,z;

  if (ampl_im>1.0)
    {
      x=((double) xindex + 0.5  - 0.5*g.ngps_x())/(0.5*g.ngps_x())
	*((double) xindex + 0.5  - 0.5*g.ngps_x())/(0.5*g.ngps_x());
      
      switch(g.dimens())
	{
	case 3 : 
	  y=((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y())
	    *((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y());
	  z=((double) zindex + 0.5  - 0.5*g.ngps_z())/(0.5*g.ngps_z())
	    *((double) zindex + 0.5  - 0.5*g.ngps_z())/(0.5*g.ngps_z());
	  return ampl_im*(x*x*x*x*x*x*x*x+y*y*y*y*y*y*y*y+z*z*z*z*z*z*z*z);
	  break;
	  
	case 2 : case 6 :
	  y=((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y())
	    *((double) yindex + 0.5  - 0.5*g.ngps_y())/(0.5*g.ngps_y());
	  return ampl_im*(x*x*x*x*x*x*x*x+y*y*y*y*y*y*y*y);
	  break;
	  
	case 1 : case 5 : case 15 :
	  return ampl_im*(x*x*x*x*x*x*x*x);
	  break;
	  
	case 4 : case 14 : case 24 : case 34 : case 44 :		
	  x=((double) xindex + 0.5  )/(g.ngps_x())
	    *((double) xindex + 0.5  )/(g.ngps_x());
	  return ampl_im*(x*x*x*x*x*x*x*x);
	  break;
	  
	default : return 0.0;
	}
    }
  else
    {
      return 0.0;
    };
      
}  


void project_output(FILE* os, wavefunction &wf)
{
  long i;
  for (i=0; i<wf.wf_size(); i++)
    {
      fprintf(os,"%e ",real(wf[i]*conj(wf[i])));
    };

}


// overloaded below
void observables_output(grid g, FILE* os, 
			const wavefunction &orb_energs, 
			const wavefunction &orb_hartrees, 
			const fluid &orb_norms)
{
  long no;

  for (no=0; no<g.ngps_z(); no++)
    {
      fprintf(os,"%16.12e %16.12e ",real(orb_energs[no])+ real(orb_hartrees[no]),orb_norms[no]);
    };

}

// overloaded above
void observables_output(grid g, FILE* os, 
			const wavefunction &orb_energs, 
			const wavefunction &orb_hartrees,
			const wavefunction &orb_fields,
			const fluid &orb_norms)
{
  long no;

  for (no=0; no<g.ngps_z(); no++)
    {
      fprintf(os,"%16.12e %16.12e ",real(orb_energs[no])+ real(orb_hartrees[no])+ real(orb_fields[no]),orb_norms[no]);
    };

}
